% Function to compute the director profile of a liquid crystal cell for a
% beam coupling geometry with decoupled electric field.  
%
% The code uses the approximate Q tensor equations:
%
% [1] K.R. Daly, G. D’Alessandro, and M. Kaczmarek. 
%     A fast and efficient non-stiff Q-tensor based algorithm for liquid 
%     crystal alignment. 
%
% To be published in SIAM Journal on Applied Mathematics,2010.
%
% Last edited 25/06/10
%
% Initialise
clear; clc;
V_save = 5;         % voltage
dt = 1e-3;          % time step 
tol_dadt = 1e-3;    % convergence criteria
% Number of discrete X coordinates and discrete Z coordinates
Nx = 14;    Nz = 14;     
% cell size parameters
Lz = 12e-6;   % cell depth
Lx = 12e-6;   % period of applied voltage
Beta = Lx/Lz; % cell size ratio for one period of applied voltage
% define physical constants
A = 0.13e6; B = 1.6e6;  C = 3.9e6;
K = 20e-12;  % elastic constant
e_par = 9.1;  % parallel dielectric permittivity  
e_per = 4.1;  % perpendicular dielectric permittivity
e_u = (e_par+2*e_per)/3; delta_e = (e_par-e_per);   e_0 = 8.85e-12;
% non-dimensional constants as defined in Section 2 of [1]
phi_0 = 1;
T_0 = -10;
gamma_1 = 0.081;
S = 0.25*(3+sqrt(9-8*T_0));
S_hat = 2*B/(3*C)*S;
L2 = K/(3*S_hat^2);
xi_0 = 9*C/(2*B^2)*L2/(Lx^2);
chi_a = (9*e_0*delta_e*C^2*phi_0^2)/(2*Lx^2*B^3);
zeta = gamma_1/(3*S_hat^2);
tau_d = 9*C/(2*B^2)*zeta;
%
% create grid and differentiation matrices
%
[Dz,z] = cheb(Nz);  z = (z+1)/2;    Dz = 2*Dz;  Nz = Nz+1;
% create first and second order x axis periodic differentiation matrix
h = Beta/Nx;    x = h*(0:Nx-1)';
col = [0 0.5.*(-1).^(1:Nx-1).*cot((1:Nx-1)*pi*h/Beta)];
Dx = 2*pi/Beta*toeplitz(col,col([1 Nx:-1:2]));  
% build 2d differentiation matrices in x and z 
DX = kron(Dx,eye(Nz));      DZ = kron(eye(Nx),Dz);
Lap = DX^2+DZ^2;  % Laplacian
% create grid
[X,Z] = meshgrid(x,z);
%
% define boundary conditions at pvk and pi layers, note the pvk
% boundary is defined as being z=0 and the pi boundary is at z=1, 
% we define the boundary conditions in terms of the [theta,phi] 
% coordinate system as defined in [1]
%
theta_pvk = 0*ones(Nx,1);
psi_pvk = pi/2*ones(Nx,1);
theta_pi = pi/180*ones(Nx,1);
psi_pi = pi/2*ones(Nx,1);
V_PVK = (0.0+1.0*sin(pi*x/Beta).^2);
V_PI =  zeros(size(V_PVK));
% start from random initial conditions
theta = [theta_pi.';pi/2*rand(Nz-2,Nx);theta_pvk.'];
psi = [psi_pi.';pi/2*rand(Nz-2,Nx);psi_pvk.'];
% components
a(:,:,1) = -1/2*S*(3*(cos(theta).^2)-2);
a(:,:,2) = 1/2*sqrt(3)*S*cos(theta).^2.*(2*cos(psi).^2-1);
a(:,:,3) = sqrt(3)*S*cos(theta).^2.*cos(psi).*sin(psi);
a(:,:,4) = sqrt(3)*S*sin(2*theta)/2.*cos(psi);
a(:,:,5) = sqrt(3)*S*sin(2*theta)/2.*sin(psi);
LocPI = 1:Nz:Nx*Nz;    LocPVK = Nz:Nz:Nx*Nz;    Loc = [LocPVK,LocPI];
Sp = zeros(Nx*Nz,1); Sp(LocPVK) = V_PVK;  Sp(LocPI) = V_PI;
% Define I, variable used for rapid conversion between component and
% director notations
I = zeros(Nx*(Nz-2),3,3);   I(:,1,1) = 1/3; I(:,2,2) = 1/3; I(:,3,3) = 1/3;
%
% define initial conditions away from boundaries
%
a_pi = squeeze(a(1,:,:));
a_pvk = squeeze(a(end,:,:));
a_o = a(2:end-1,:,:); 
%
% calculate electric field components in cell (solve div.D=0)
%
D = Lap;
D(Loc,:)=0;   D(Loc,Loc) = eye(Nx*2);  
phi_scale = V_save*(D\Sp); 
dx_p = DX*phi_scale; dz_p = DZ*phi_scale;
% calculate components of electrostatic tensor 
e = zeros(length(dx_p),5);
e(:,1) = -0.5.*dx_p.^2+dz_p.^2;
e(:,2) = sqrt(3)./2.*dx_p.^2;
e(:,4) = (dz_p.*dx_p).*sqrt(3);
%
% Solve liquid crystal equations using MATLAB's inbuilt function ode113
%
diff = 1;
options = odeset('RelTol',1e-9,'AbsTol',1e-9);
warning off MATLAB:divideByZero;
T = 0;
while diff>=tol_dadt    % repeat till da_dt is smaller than tol_dadt
    [T,Y] = ode113(@comp,[T(end),T(end)+dt],a_o(:),options,...
        Lap,a_pi,a_pvk,Nx,Nz,xi_0,chi_a,S,I,e);  
    % calculate time derivative
    diff = norm(Y(end,:).'-a_o(:))./dt;
    % rescale time for output
    t_dim = T(end)*3*S^2*xi_0/tau_d;
    disp(['da/dt = ',num2str(diff,'%6.4f'),' at t = '...
        ,num2str(t_dim,'%6.4f')]);
    a_o = Y(end,:);
    a_o = reshape(a_o(:),Nz-2,Nx,5);
%
% plot of time evolution of components, comment line 109-130 to turn off
%
% add boundary elements
    a_d(:,:) = [a_pi(:).';a_o(:,:);a_pvk(:)'];
    a_d = reshape(a_d,Nz,Nx,5);
    a1_d = a_d(:,:,1);
    a2_d = a_d(:,:,2);
    a3_d = a_d(:,:,3);
    a4_d = a_d(:,:,4);
    a5_d = a_d(:,:,5);
% plot
    ad = reshape(a_d,[],5);
    figure(1); 
    subplot(2,3,1); surf(Z*Lz,X*Lx/Beta,reshape(a1_d,Nz,Nx)); 
    title('a1');    axis([0 Lz 0 Lx -S S]);
    subplot(2,3,2); surf(Z*Lz,X*Lx/Beta,reshape(a2_d,Nz,Nx)); 
    title('a2');    axis([0 Lz 0 Lx -S S]);
    subplot(2,3,3); surf(Z*Lz,X*Lx/Beta,reshape(a3_d,Nz,Nx)); 
    title('a3');    axis([0 Lz 0 Lx -S S]);
    subplot(2,3,4); surf(Z*Lz,X*Lx/Beta,reshape(a4_d,Nz,Nx)); 
    title('a4');    axis([0 Lz 0 Lx -S S]);
    subplot(2,3,5); surf(Z*Lz,X*Lx/Beta,reshape(a5_d,Nz,Nx)); 
    title('a5');    axis([0 Lz 0 Lx -S S]);
    subplot(2,3,6); surf(Z*Lz,X*Lx/Beta,reshape(phi_scale,Nz,Nx)); 
    title('V');     axis([0 Lz 0 Lx 0 max(V_save,eps)]);
end
%
% create 2D and 3D director plots
%
% create uniform n_uni*n_uni grid
n_uni = 16;                         
x_uni = linspace(0,Beta,n_uni);
z_uni = linspace(0,1,n_uni);
[X,Z] = meshgrid([x; Beta],z);
[X_uni,Z_uni] = meshgrid(x_uni,z_uni);
% reshape component field for plots
a1_d = reshape(a_d(:,:,1),Nz,Nx);    
a2_d = reshape(a_d(:,:,2),Nz,Nx);
a3_d = reshape(a_d(:,:,3),Nz,Nx);
a4_d = reshape(a_d(:,:,4),Nz,Nx);
a5_d = reshape(a_d(:,:,5),Nz,Nx);
% add additional point in x dir
a1 = [a1_d a1_d(:,1)];              
a2 = [a2_d a2_d(:,1)];
a3 = [a3_d a3_d(:,1)];
a4 = [a4_d a4_d(:,1)];
a5 = [a5_d a5_d(:,1)];
% Interpolate onto uniform grid
a_u(:,:,1) = griddata(Z,X,a1,Z_uni,X_uni,'cubic');   
a_u(:,:,2) = griddata(Z,X,a2,Z_uni,X_uni,'cubic');
a_u(:,:,3) = griddata(Z,X,a3,Z_uni,X_uni,'cubic');
a_u(:,:,4) = griddata(Z,X,a4,Z_uni,X_uni,'cubic');
a_u(:,:,5) = griddata(Z,X,a5,Z_uni,X_uni,'cubic');
% convert to director field
Q_u = Comp2Tens(a_u);
n_u = Tens2Eig(Q_u);
n(:,:,1) = squeeze(n_u(:,:,1,3));
n(:,:,2) = squeeze(n_u(:,:,2,3));
n(:,:,3) = squeeze(n_u(:,:,3,3));
% interpolate potential onto fine grid
n_uni = 100;
xx = linspace(0,Lx,n_uni);
zz = linspace(0,Lz,n_uni);
[XX,ZZ] = meshgrid(xx,zz);
phi_d = phi_scale;
phi_d = reshape(phi_d,Nz,Nx);
V = interp2(X,Z,[phi_d phi_d(:,1)], XX/Lx,ZZ/Lz,'cubic');
X_uni = X_uni*Lx;
Z_uni = Z_uni*Lz;
% plot liquid crystal director field
figure(2); clf;
%3D plot
subplot(1,10,5:10); hold on;
quiver3(X_uni/Lx,Z_uni/Lz,zeros(size(X_uni)),...
    n(:,:,1),n(:,:,3),n(:,:,2),0.2,'.k','linewidth',3);
quiver3(X_uni/Lx,Z_uni/Lz,zeros(size(X_uni)),...
    -n(:,:,1),-n(:,:,3),-n(:,:,2),0.2,'.k','linewidth',3);
% plot potential
surf(XX/Lx,ZZ/Lz,zeros(size(XX)),V,'edgecolor','none');
colormap hot
alpha(.4)
hold off;   axis([0 Beta 0 1 -1/4 1/4]);
set(gca,'xtick',[],'fontsize',16);
set(gca,'ytick',[],'fontsize',16);
set(gca,'ztick',[],'fontsize',16);
set(gca,'Xcolor',[1 1 1]);
set(gca,'Ycolor',[1 1 1]);
set(gca,'Zcolor',[1 1 1]);
%2D plot
subplot(5,10,[11:14 21:24 31:34]); hold on;
quiver(X_uni/1e-6,Z_uni/1e-6,...
    n(:,:,1),n(:,:,3),0.25,'.k','linewidth',3);
quiver(X_uni/1e-6,Z_uni/1e-6,...
    -n(:,:,1),-n(:,:,3),0.25,'.k','linewidth',3);
% plot potential
surf(XX/1e-6,ZZ/1e-6,V-max(max(V)),'edgecolor','none');
colormap hot
alpha(.4)
xlabel('Distance along cell \mum','fontsize',18);
ylabel('Distance through cell \mum','fontsize',18);
set(gca,'xtick',get(gca,'xtick'),'fontsize',16);
hold off;   axis([0 Beta*Lx*1e6 0 Lz*1e6]); 
warning on all
