% function to compute da/dt of the order 1 equations based on inputs
% for the simplest case, no electric field coupling, one elastic constant
%
% [1] K.R. Daly, G. D’Alessandro, and M. Kaczmarek. 
%     A fast and efficient non-stiff Q-tensor based algorithm for liquid 
%     crystal alignment. 
%
function [da_dt] = comp(t,a_o,Lap,a_pi,a_pvk,Nx,Nz,xi_0,chi_a,S,I,e)
a_o = reshape(a_o(:),Nz-2,Nx*5);
% add boundary elements
a_d(:,:) = [a_pi(:).';a_o(:,:);a_pvk(:)'];
a_o = reshape(a_o,[],5);
a_d = reshape(a_d,[],5);
%
% Calculate right hand side of equations F_n(a_p) in notes
%
% calculate source term
Source = reshape((Lap*a_d)+chi_a./xi_0.*e,Nz,Nx*5);
Source = reshape(Source(2:end-1,:),[],5);
% create spanning vectors equations (4.14) in [1]
SpaV(:,:,3) = [+sqrt(3).*a_o(:,5),     +a_o(:,5),   -a_o(:,4),   +a_o(:,3),                   -(sqrt(3).*a_o(:,1)+a_o(:,2))];
SpaV(:,:,1) = [zeros(size(a_o(:,1))), -2.*a_o(:,3), +2.*a_o(:,2), -a_o(:,5),                    +a_o(:,4)];
SpaV(:,:,2) = [-sqrt(3).*a_o(:,4),     +a_o(:,4),   +a_o(:,5),   (sqrt(3).*a_o(:,1)-a_o(:,2)), -a_o(:,3)];
% calculate time derivatives, equation (4.19a) & (4.19b) in [1]
eq1 = sum(SpaV(:,:,1).*Source,2); 
eq2 = sum(SpaV(:,:,2).*Source,2);
eq3 = sum(SpaV(:,:,3).*Source,2);
%
% Calculate theta_n, phi_n from components
%
% convert to director notation to define coordinate systems
q = sum(sqrt(2/3)./S.*Comp2Tens(a_o)+I,3);
n = diag(1./sqrt(sum(q.^2,2)))*q;
% Criteria for coordinate system choice
c1=abs(n(:,1))<=4./(3*sqrt(2));   %use coord system theta' phi'
c2=abs(n(:,3))<=4./(3*sqrt(2));   %use coord system theta  phi
% calculate intermediate derivatives in each coordinate system
dphi_dt1 = c2.*(eq1./(1-n(:,3).^2+not(c2)));
dphi_dt2 = c1.*(-eq3./(1-n(:,1).^2+not(c1)));
phi1 = c2.*(atan((n(:,2)+not(c2))./n(:,1))+(n(:,1)<0)*pi);
phi2 = c1.*(atan((n(:,2)+not(c1))./n(:,3))+(n(:,3)<0)*pi);
sp1 = sin(phi1); cp1 = cos(phi1);
sp2 = sin(phi2); cp2 = cos(phi2);
dtheta_dt1 = -sp1.*(eq3)+cp1.*(eq2);
dtheta_dt2 = -sp2.*(eq1)+cp2.*(eq2);
% finally calculate dadt
da_dt1 = (repmat(dphi_dt1,1,5).*SpaV(:,:,1)-...
                 repmat(dtheta_dt1.*sp1,1,5).*SpaV(:,:,3)+...
                 repmat(dtheta_dt1.*cp1,1,5).*SpaV(:,:,2));
da_dt2 = (repmat(-dphi_dt2,1,5).*SpaV(:,:,3)+...
                 repmat(dtheta_dt2.*cp2,1,5).*SpaV(:,:,2)-...
                 repmat(dtheta_dt2.*sp2,1,5).*SpaV(:,:,1));
% select coordinate system
da_dt = repmat(c1&c2,1,5).*(da_dt1+da_dt2)./2+... % both coord systems
        repmat(c1&not(c2),1,5).*da_dt2+...        % c1 only
        repmat(c2&not(c1),1,5).*da_dt1;           % c2 only
da_dt = reshape(da_dt(:),[],1); % reshape output
end